(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  odstranny podmnn peklady pro MS-DOS (P92)
2)  pechod na 32 bit: odstrann longint a redefinice typu integer (P92)
3)  pechod na 32 bit: word zmnn na integer, doplnn parametr Color (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

UNIT DrvTypes;  {Spolecne deklarace pro generator a DLL driver}

INTERFACE

TYPE
  Capability_T = (
    BitMapped,      {vystup muze byt po pasech}
    Optimizable,    {neni-li pritomen, je zbytecne optimalizovat}
    DrawLines,
    DrawArcs,
    FillBoxes,
    FillCircles,
    DrawTexts,      {napr. do DXF}
    CzechTexts,     {text smi obsahovat ceske znaky - pouze je-li DrawTexts}
    FlashPads,
    DrillHoles,
    UserTools,      {uzivatelsky definovatelne nastroje - ve chvili pouziti}
    ConfigTools);   {moznost zmenit tabulku nastroju konfig. souborem}

  Capabilities_T = set of Capability_T;

  DriverUnits_T = integer;  {jednotky ciloveho zarizeni, velikost definovana jinde}

  ToolShape_T = (
    CircleTool,    {Height nehraje roli}
    SquareTool,    {Height nehraje roli}
    OvalTool,
    RectangTool,
    AnnulusTool,   {Height obsahuje sirku prstence}
    ThermalTool,   {dtto}
    DiamondTool,   {dtto, Width je uhlopricka}
    SpecialTool);  {Height nehraje roli}

  Thousandth_T = integer;

  ToolUsage_T = (
    UsageDraw,
    UsageFlash,
    UsageDrill);
  ToolUsages_T = set of ToolUsage_T;

  ToolCode_T = integer;

  ToolDescr_T = packed record
                  DescrStr: string [30];
                  Shape:    ToolShape_T;
                  Width,
                  Height:   Thousandth_T;
                  Usage:    ToolUsages_T;
                  Code:     ToolCode_T;
                end;

  ToolTablePtr_T = ^ ToolTable_T;
  ToolTable_T    = packed array [0..200] of ToolDescr_T;
  ToolNumber_T   = integer;

  DriverDescr_T = packed record
                    Name:           string [24];
                    MajorVersion,
                    MinorVersion:   byte;           {nebo do jmena?}
                    Capabilities:   Capabilities_T;
                    NoOfParams:     integer;
                    FilesName:      string [8];
                    TargetExt:      string [4];
                    NoOfTools:      integer;        {muze byt i 0}
                    InitialTools:   ^ToolDescr_T;   {muze byt i nil}
                    MaxNoOfTools:   integer;
                    ToolTablePtr:   ToolTablePtr_T;
                    X0, Y0, X1, Y1: integer;        {mm}
                    XDividend,
                    YDividend,
                    Divisor:        integer;
                    Granularity:    integer;        {modularita bitmapy}
                  end;
  DescrPtr_T = ^ DriverDescr_T;

  ParamPtr_T = ^ ParamDescr_T;
  ParamType_T = (_Numeric, _Explicite, _ChangeCapability);
  ParamDescr_T = packed record
                   DescrStr: string [30];
                   Value:    integer;
                   case ParamType: ParamType_T of
                     _Numeric:   (MinVal, MaxVal:  integer);
                     _Explicite: (Values: string);
                 end;
  ParamNumber_T = integer;

  BitmapRow_T    = array [0..0] of byte;
  BitmapRowPtr_T = ^ BitmapRow_T;
  Bitmap_T       = array [0..9999] of BitmapRowPtr_T;
  Planes_T       = array [0..7] of ^ Bitmap_T;


  ColorsStr_T = string [7];

  GetColors_T = FUNCTION: ColorsStr_T;


  WriteProc_T = FUNCTION (var V; Length: integer): Boolean;


  GetDescrPtr_T    = FUNCTION: DescrPtr_T;
  {vraci primo ukazatel na pole parametru}

  GetParamPtr_T    = FUNCTION (ParamNumber: ParamNumber_T): ParamPtr_T;
  {ParamNumber je v rozsahu 0..NoOfParams - 1}

  GetToolsPtr_T    = FUNCTION: ToolTablePtr_T;
  {vraci primo ukazatele na pracovni tabulku nastroju}

  SetParamValue_T  = PROCEDURE (ParamNumber: ParamNumber_T; ParamValue: integer);

  OpenEquip_T      = PROCEDURE (WriteProc: WriteProc_T;
                                X0, Y0, X1, Y1: DriverUnits_T);
  {vyuzite rozmery kreslici plochy, jsou-li znamy}

  Comment_T        = PROCEDURE (const CommentString: string);

  OutputBitmap_T   = PROCEDURE (BitmapWidth, BitmapHeight, NumOfPlanes: integer;
                                const Planes: Planes_T);

  OpenLayer_T      = PROCEDURE (Layer: integer; Color: integer = 0);

  DrawLine_T       = PROCEDURE (X0, Y0, X1, Y1: DriverUnits_T;
                                ToolNumber: ToolNumber_T);

  DrawArc_T        = PROCEDURE (X, Y, R: DriverUnits_T;
                                Angle1, Angle2: integer;  {deg}
                                ToolNumber: ToolNumber_T);
  {priklady: Angle1, Angle2 =  90, 180 ... 1 kvadrant CCW}
  {                         =   0, -90 ... 1 kvadrant CW}
  {                         =   0, 270 ... 3 kvadranty CCW}
  {                         = 270,   0 ... 3 kvadranty CW}

  FillBox_T        = PROCEDURE (X1, Y1, X2, Y2: DriverUnits_T;
                                ToolNumber: ToolNumber_T);

  FillCircle_T     = PROCEDURE (X, Y, R: DriverUnits_T;
                                ToolNumber: ToolNumber_T);

  DrawText_T       = PROCEDURE (const TextStr: string;
                                X, Y, W, H: DriverUnits_T;
                                Angle: integer;           {deg}
                                Mirroring: Boolean;
                                ToolNumber: ToolNumber_T);

  FlashPad_T       = PROCEDURE (X, Y: DriverUnits_T;
                                ToolNumber: ToolNumber_T);

  DrillHole_T      = PROCEDURE (X, Y: DriverUnits_T;
                                ToolNumber: ToolNumber_T);

  CloseEquip_T     = PROCEDURE;

{mezi volanim OpenEquip a CloseEquip nebude volana zadna procedura Get...}

IMPLEMENTATION

END.
