(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  parametry procedury OutputBitmap zmnny na typ integer (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY Epson;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    Resolution,
    LeftMargin,
    UpperMargin,
    EjectPage);

  Resolution_T = (Res60by60,  Res80by60,   Res90by60,   Res120by60,   Res240by60,
                  Res60by180, Res90by180,  Res120by180, Res180by180,  Res360by180,
                                                        Res180by360,  Res360by360);

CONST
  ToolCount = 0;

  Granularity = 48;

  DriverDescr: DriverDescr_T = (
    Name:         'Epson (CMYK)';
    MajorVersion: 1;
    MinorVersion: 0;
    Capabilities: [BitMapped, UserTools, ConfigTools];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'Epson';
    TargetExt:    'PRN';
    NoOfTools:    ToolCount;
    InitialTools: nil;
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           -2;
    Y0:           -2;
    X1:           195;
    Y1:           280;
    XDividend:    3600;
    YDividend:    3600;
    Divisor:      254;
    Granularity:  Granularity);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  '>Resolution (dpi)';
    Value:     Ord (Res360by360);
    ParamType: _Explicite;
    Values:    ' >60 by  60| >80 by  60| >90 by  60|120 by  60|>240 by  60|'
             + ' 60 by 180| 90 by 180|120 by 180|>180 by 180|360 by 180|'
             + '180 by 360|>360 by 360'),
   (DescrStr:  '>Left Margin (mm)';
    Value:     0;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    200),
   (DescrStr:  '>Upper Margin (mm)';
    Value:     0;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    200),
   (DescrStr:  '>Eject Page';
    Value:     Ord (false);
    ParamType: _Explicite;
    Values:    'Off|On'));

// -----------------------------------------------------------------------------

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetColors: ColorsStr_T;
begin
  GetColors := 'KCMY';
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

// -----------------------------------------------------------------------------

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
  var M, I: integer;
begin
  Write := WriteProc;
  if Params [UpperMargin].Value > 0 then begin
    M := 180 * 10 * longint (Params [UpperMargin].Value) div 254;
    for I := 1 to M div 250 do
      WriteStr (#27'J'#250);  {perform 250/180" line feed}
    WriteStr (#27'J');
    WriteByte (M mod 250);
  end;
end;

PROCEDURE OutputBitmap (BitmapWidth, BitmapHeight, NumOfPlanes: integer; const Planes: Planes_T);
  label
    UsedColumn;
  const
    ResCodes:   array [Resolution_T] of byte = (0,  4,  6,  1,  3,
                                               32, 38, 33, 39, 40,
                                                           71, 72 {, 73});
    BytesTable: array [Resolution_T] of byte = (1,  1,  1,  1,  1,
                                                3,  3,  3,  3,  3,
                                                            6,  6 {,  6});
    ColorCodes: array [0..7] of byte = (0, 2, 1, 4, 0, 0, 0, 0);
  var
    BytesPerColumn, Plane, ActualWidth, Row, Column, Bit, I, R: integer;
    B, M: byte;

begin
  BytesPerColumn := BytesTable [Resolution_T (Params [Resolution].Value)];
  Row := 0;
  while Row <= BitmapHeight - 8 * BytesPerColumn do begin
    for Plane := 0 to NumOfPlanes - 1 do begin
      ActualWidth := BitmapWidth;
      while ActualWidth > 0 do begin
        for R := 0 to 8 * BytesPerColumn - 1 do
          if Planes [Plane]^ [Row + R]^ [ActualWidth - 1] <> 0 then goto UsedColumn;
        Dec (ActualWidth);
      end;
UsedColumn:
      if ActualWidth > 0 then begin
        WriteStr (#27'r');
        WriteByte (ColorCodes [Plane]);
        WriteStr (#13);
        if Params [LeftMargin].Value > 0 then begin
          WriteStr (#27'$');  {set absolute dot position}
          WriteWord (60 * 10 * longint (Params [LeftMargin].Value) div 254);
        end;
        WriteStr (#27'*');
        WriteByte (ResCodes [Resolution_T (Params [Resolution].Value)]);
        WriteWord (ActualWidth * 8);
        for Column := 0 to ActualWidth - 1 do
          for Bit := 7 downto 0 do begin
            M := 1 shl Bit;
            for I := 0 to BytesPerColumn - 1 do begin
              B := 0;
              for R := 0 to 7 do
                if (Planes [Plane]^ [Row + 8 * I + R]^ [Column] and M) <> 0 then
                  B := B or ($80 shr R);
              WriteByte (B);
            end;
          end;
      end;
    end;
    WriteStr (#27'J'#24);  {8/60" line feed}
    Inc (Row, 8 * BytesPerColumn);
  end;
end {OutputBitmap};

PROCEDURE CloseEquip;
begin
  WriteStr (#13);         {carriage return}
  WriteStr (#27'r'#0);    {restores black}
  if Boolean (Params [EjectPage].Value) then
    WriteStr (^L);        {form feed}
end;

EXPORTS
  GetDescrPtr440,
  GetColors,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OutputBitmap,
  CloseEquip;

BEGIN
END.
