(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  nov hodnota parametru Excellon Format dovoluje pipojit tabulku vrtk na
    konec vstupnho souboru (T91)
2)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
3)  do funkce OpenLayer doplnn parametr Color (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY Excellon;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    ExcellonFormat,
    NoOfDigits,
    TrailingZeroes,
    RepeatedCoordinates,
    RepeatCodesEnabled,
    OutputCode,
    NewLineCode,
    EvenParityEnabled,
    BlankTapeLength);

  ExcellonFormat_T = (Standard, Extended, ExtTable);
  OutputCode_T     = (AsciiCode, EiaCode);
  NewLineCode_T    = (Lf, CrLf);

CONST
  ToolCount = 14;
  ToolTable: array [1..ToolCount] of ToolDescr_T = (
    (DescrStr: 'T1  (0.6  mm)'; Shape: CircleTool; Width:  600; Height: 0; Usage: [UsageDrill]; Code:  1),
    (DescrStr: 'T2  (0.7  mm)'; Shape: CircleTool; Width:  700; Height: 0; Usage: [UsageDrill]; Code:  2),
    (DescrStr: 'T3  (0.8  mm)'; Shape: CircleTool; Width:  800; Height: 0; Usage: [UsageDrill]; Code:  3),
    (DescrStr: 'T4  (0.9  mm)'; Shape: CircleTool; Width:  900; Height: 0; Usage: [UsageDrill]; Code:  4),
    (DescrStr: 'T5  (1.0  mm)'; Shape: CircleTool; Width: 1000; Height: 0; Usage: [UsageDrill]; Code:  5),
    (DescrStr: 'T6  (1.1  mm)'; Shape: CircleTool; Width: 1100; Height: 0; Usage: [UsageDrill]; Code:  6),
    (DescrStr: 'T7  (1.2  mm)'; Shape: CircleTool; Width: 1200; Height: 0; Usage: [UsageDrill]; Code:  7),
    (DescrStr: 'T8  (1.3  mm)'; Shape: CircleTool; Width: 1300; Height: 0; Usage: [UsageDrill]; Code:  8),
    (DescrStr: 'T9  (1.5  mm)'; Shape: CircleTool; Width: 1500; Height: 0; Usage: [UsageDrill]; Code:  9),
    (DescrStr: 'T10 (1.8  mm)'; Shape: CircleTool; Width: 1800; Height: 0; Usage: [UsageDrill]; Code: 10),
    (DescrStr: 'T11 (2.0  mm)'; Shape: CircleTool; Width: 2000; Height: 0; Usage: [UsageDrill]; Code: 11),
    (DescrStr: 'T12 (2.2  mm)'; Shape: CircleTool; Width: 2200; Height: 0; Usage: [UsageDrill]; Code: 12),
    (DescrStr: 'T13 (2.6  mm)'; Shape: CircleTool; Width: 2600; Height: 0; Usage: [UsageDrill]; Code: 13),
    (DescrStr: 'T14 (3.15 mm)'; Shape: CircleTool; Width: 3150; Height: 0; Usage: [UsageDrill]; Code: 14));

  MaxToolCount = 200;

VAR
  WorkToolTable: array [0..MaxToolCount] of ToolDescr_T;
  ToolCounts:    array [0..MaxToolCount] of integer;

CONST
  DriverDescr: DriverDescr_T = (
    Name:         'Excellon 4.30 PH 2002';
    MajorVersion: 4;
    MinorVersion: 30;
    Capabilities: [Optimizable, DrillHoles];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'Excellon';
    TargetExt:    '.Exc';  {tady?}
    NoOfTools:    ToolCount;
    InitialTools: @ ToolTable [1];
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           0;
    Y0:           0;
    X1:           800;
    Y1:           600;
    XDividend:    100;
    YDividend:    100;
    Divisor:      1);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  'Excellon >Format';
    Value:     Ord (Standard);
    ParamType: _Explicite;
    Values:    '>Standard|>Extended|Extended  +  >Table'),
   (DescrStr:  '>Number of Digits';
    Value:     5;
    ParamType: _Numeric;
    MinVal:    1;
    MaxVal:    8),
   (DescrStr:  '>Trailing Zeroes';
    Value:     Ord (true);
    ParamType: _Explicite;
    Values:    '>Omitted|>Included'),
   (DescrStr:  '>Repeated Coordinates';
    Value:     Ord (false);
    ParamType: _Explicite;
    Values:    '>Omitted|>Included'),
   (DescrStr:  'Repeat (R) >Codes';
    Value:     Ord (false);
    ParamType: _Explicite;
    Values:    '>Disabled|>Enabled'),
   (DescrStr:  '>Output Code';
    Value:     Ord (AsciiCode);
    ParamType: _Explicite;
    Values:    '>ASCII|>EIA'),
   (DescrStr:  'New >Line Code';
    Value:     Ord (CrLf);
    ParamType: _Explicite;
    Values:    '>LF|>CR-LF'),
   (DescrStr:  '>Even Parity';
    Value:     Ord (false);
    ParamType: _Explicite;
    Values:    'Off|On'),
   (DescrStr:  '>Blank Tape Length';
    Value:     0;  {200}
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    1000));

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

// -----------------------------------------------------------------------------

CONST
  RejectR1Code  = true;
  MaxRepeatCode = 99;   {standard format only, 9999 for extended format}

VAR
  CurrentX, CurrentY,
  IncrementX, IncrementY: DriverUnits_T;
  CurrentToolCode:        ToolCode_T;
  IncrementCount:         integer;
  FirstHole:              Boolean;


procedure Output (const AsciiString: string);
  const
    EiaSpace = $10;     Eia0     = $20;     EiaMinus = $40;     EiaPlus  = $70;
    Eia1     = $01;     EiaSlash = $31;     EiaJ     = $51;     EiaA     = $61;
    Eia2     = $02;     EiaS     = $32;     EiaK     = $52;     EiaB     = $62;
    Eia3     = $13;     EiaT     = $23;     EiaL     = $43;     EiaC     = $73;
    Eia4     = $04;     EiaU     = $34;     EiaM     = $54;     EiaD     = $64;
    Eia5     = $15;     EiaV     = $25;     EiaN     = $45;     EiaE     = $75;
    Eia6     = $16;     EiaW     = $26;     EiaO     = $46;     EiaF     = $76;
    Eia7     = $07;     EiaX     = $37;     EiaP     = $57;     EiaG     = $67;
    Eia8     = $08;     EiaY     = $38;     EiaQ     = $58;     EiaH     = $68;
    Eia9     = $19;     EiaZ     = $29;     EiaR     = $49;     EiaI     = $79;
    EiaEor   = $0B;     EiaComma = $3B;                         EiaPeriod = $6B;

    EiaEob   = $80;   { EiaEor <=> ASCII "%",   EiaEob <=> ASCII Ctrl-J or ";" }

    EiaConvertTable: array [char] of byte = (
{0}   EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaEob,   EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{10}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{20}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaEor,   EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaPlus,  EiaComma, EiaMinus, EiaPeriod,EiaSlash,
{30}  Eia0,     Eia1,     Eia2,     Eia3,     Eia4,     Eia5,     Eia6,     Eia7,
      Eia8,     Eia9,     EiaSpace, EiaEob,   EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{40}  EiaSpace, EiaA,     EiaB,     EiaC,     EiaD,     EiaE,     EiaF,     EiaG,
      EiaH,     EiaI,     EiaJ,     EiaK,     EiaL,     EiaM,     EiaN,     EiaO,
{50}  EiaP,     EiaQ,     EiaR,     EiaS,     EiaT,     EiaU,     EiaV,     EiaW,
      EiaX,     EiaY,     EiaZ,     EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{60}  EiaSpace, EiaA,     EiaB,     EiaC,     EiaD,     EiaE,     EiaF,     EiaG,
      EiaH,     EiaI,     EiaJ,     EiaK,     EiaL,     EiaM,     EiaN,     EiaO,
{70}  EiaP,     EiaQ,     EiaR,     EiaS,     EiaT,     EiaU,     EiaV,     EiaW,
      EiaX,     EiaY,     EiaZ,     EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,

{80}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{90}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{A0}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{B0}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{C0}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{D0}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{E0}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
{F0}  EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace,
      EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace, EiaSpace);

    Cr:  char = ^M;
    CrP: char = char (byte (^M) or $80);

  var i: integer;

begin
  if OutputCode_T (Params [OutputCode].Value) = EiaCode then
    for i := 1 to Length (AsciiString) do
      WriteByte (EiaConvertTable [AsciiString [i]])
  else
    for i := 1 to Length (AsciiString) do begin
      if (AsciiString [i] = ^J)
        and (NewLineCode_T (Params [NewLineCode].Value) = CrLf)
      then
        if Boolean (Params [EvenParityEnabled].Value) then
          WriteByte (Ord (CrP))
        else
          WriteByte (Ord (Cr));
      if Boolean (Params [EvenParityEnabled].Value) then
        WriteByte (Ord (EvenParity (AsciiString [i])))
      else
        WriteByte (Ord (AsciiString [i]));
    end;
end {Output};

// --- TAPE HEADER -------------------------------------------------------------

function RealStr (r: real): string;
  var s: string;
begin
  Str (r: 0: 2, s);
  RealStr := s;
end;

procedure OutputStandardTapeHeader;
  var i: integer;
begin
  for i := 1 to Params [BlankTapeLength].Value do
    Output (' ');
  Output ('%'^J);
end;

procedure OutputExtendedTapeHeader;
  var ToolNumber, i: integer;
begin
  for i := 1 to Params [BlankTapeLength].Value do
    Output (' ');
  Output ('M48'^J'METRIC,LZ,000.00'^J'R,T'^J);
  for ToolNumber := 1 to DriverDescr.NoOfTools do
    Output ('T' + IntStr (WorkToolTable [ToolNumber].Code)
          + 'C' + RealStr (WorkToolTable [ToolNumber].Width / 1000)
          + ^J);
  Output ('%'^J);
end;

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
begin
  Write := WriteProc;
  case ExcellonFormat_T (Params [ExcellonFormat].Value) of
    Standard: OutputStandardTapeHeader;
    Extended,
    ExtTable: OutputExtendedTapeHeader;
  end;
  CurrentToolCode := 0;
  FirstHole       := true;
  IncrementCount  := 0;
  CurrentX        := MaxLongInt;
  CurrentY        := MaxLongInt;
  CurrentToolCode := MaxInt;
  FillChar (ToolCounts, SizeOf (ToolCounts), 0);
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

PROCEDURE OpenLayer (Layer: integer; Color: integer = 0);
begin
end;

// --- OUTPUT OF COORDINATES ---------------------------------------------------

function NoTrailingZeroes (Number: longint; NumberOfDigits: integer): string;
  var WorkString: string;
begin
  WorkString := LZIntStr (Number, NumberOfDigits);
  if not Boolean (Params [TrailingZeroes].Value) then
    while (WorkString [Length (WorkString)] = '0')
      and (Length (WorkString) > 1)
    do  {remove trailing zeroes}
      Delete (WorkString, Length (WorkString), 1);
  NoTrailingZeroes := WorkString;
end;

procedure OutputRepeatCode;
begin
  if IncrementCount > 0 then begin
    if (IncrementCount = 1) and RejectR1Code then
      if IncrementX <> 0 then
        Output ('X' + NoTrailingZeroes (CurrentX + IncrementX, Params [NoOfDigits].Value) + ^J)
      else
        Output ('Y' + NoTrailingZeroes (CurrentY + IncrementY, Params [NoOfDigits].Value) + ^J)
    else begin
      case ExcellonFormat_T (Params [ExcellonFormat].Value) of
        Standard: Output ('R' + LZIntStr (IncrementCount, 2));
        Extended,
        ExtTable: Output ('R' + IntStr (IncrementCount));
      end;
      if IncrementX <> 0 then
        Output ('X' + NoTrailingZeroes (IncrementX, Params [NoOfDigits].Value) + ^J)
      else
        Output ('Y' + NoTrailingZeroes (IncrementY, Params [NoOfDigits].Value) + ^J);
    end;
    IncrementCount := 0;
    CurrentX       := MaxLongInt;  {to prevent further undefined operation}
    CurrentY       := MaxLongInt;
  end;
end;

PROCEDURE DrillHole (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);

  procedure OutputCoordinates;
  begin
    OutputRepeatCode;
    if Boolean (Params [RepeatedCoordinates].Value) or (X <> CurrentX) and (Y <> CurrentY) then
      Output ('X' + NoTrailingZeroes (X, Params [NoOfDigits].Value)
            + 'Y' + NoTrailingZeroes (Y, Params [NoOfDigits].Value) + ^J)
    else if (X = CurrentX) then
      Output ('Y' + NoTrailingZeroes (Y, Params [NoOfDigits].Value) + ^J)
    else
      Output ('X' + NoTrailingZeroes (X, Params [NoOfDigits].Value) + ^J);
    CurrentX  := X;
    CurrentY  := Y;
    FirstHole := false;
  end;

  procedure ChangeTool (ToolNumber: ToolNumber_T);
    var ToolCode: ToolCode_T;
  begin
    ToolCode := WorkToolTable [ToolNumber].Code;
    if ToolCode <> CurrentToolCode then begin
      OutputRepeatCode;
      Output ('T' + IntStr (ToolCode) + ^J);
      CurrentToolCode := ToolCode;
      FirstHole       := true;
      IncrementCount  := 0;
    end;
  end;

begin
  ChangeTool (ToolNumber);
  if IncrementCount = 0 then
    if ((X = CurrentX) or (Y = CurrentY))
      and not FirstHole and Boolean (Params [RepeatCodesEnabled].Value)
    then begin
      IncrementX := X - CurrentX;
      IncrementY := Y - CurrentY;
      IncrementCount := 1;
    end
    else
      OutputCoordinates
  else  {if IncrementCount > 0 then}
    if    (CurrentX + (IncrementCount + 1) * IncrementX = X)
      and (CurrentY + (IncrementCount + 1) * IncrementY = Y)
      and Boolean (Params [RepeatCodesEnabled].Value)
      and (IncrementCount < MaxRepeatCode)
    then
      Inc (IncrementCount)
    else
      {output repeat code, then}  OutputCoordinates;
  Inc (ToolCounts [ToolNumber]);
end {DrillHole};

function IntStrL (i, w: integer): string;
begin
  Str (i, Result);
  Result := Copy (Result + '          ', 1, w);
end;

function IntStrR (i, w: integer): string;
begin
  Str (i: w, Result);
end;

procedure OutputStatistics;
  var i, Count: integer;
begin
  Output ('M47     TOOL STATISTICS:'^J);
  Output ('M47     ======================='^J);
  Output ('M47     Tool   Count   Diameter'^J);
  Output ('M47     -----------------------'^J);
  Count := 0;
  for i := 1 to MaxToolCount do
    if ToolCounts [i] > 0 then begin
      Output ('M47     T' + IntStrL (WorkToolTable [i].Code, 4)
            + IntStrR (ToolCounts [i], 7)
            + '    ' + RealStr (WorkToolTable [i].Width / 1000) + ' mm'^J);
      Inc (Count, ToolCounts [i]);
    end;
  Output ('M47     -----------------------'^J);
  Output ('M47     Total' + IntStrR (Count, 7) + ^J);
end {OutputStatistics};

PROCEDURE CloseEquip;
begin
  OutputRepeatCode;
  case ExcellonFormat_T (Params [ExcellonFormat].Value) of
    Standard: Output ('M30'^J);
    Extended: Output ('T0'^J'M30'^J);
    ExtTable: begin
                Output ('T0'^J'M30'^J);
                OutputStatistics;
              end;
  end;
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OpenLayer,
  DrillHole,
  CloseEquip;

BEGIN
END.
