(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  parametry procedury OutputBitmap zmnny na typ integer (P93)
--------------------------------------------------------------------------------

DODLAT
1)  Ovit, kdy me petci RowBuffer
5)  dal pkazy pro LJ za elem jeho uveden do lpe definovanho mdu
    (viz str. 14-5)

----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY LaserJet;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    Resolution,
    LeftMargin,
    UpperMargin,
    EjectPage,
    Compression);

CONST
  MaxResolution = 6;
  Resolutions: array [0..MaxResolution] of integer = (75, 100, 150, 200, 300, 600, 1200);

  ToolCount = 0;

  DriverDescr: DriverDescr_T = (
    Name:         'LaserJet Printer';
    MajorVersion: 1;
    MinorVersion: 0;
    Capabilities: [BitMapped, UserTools, ConfigTools];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'LaserJet';
    TargetExt:    'PRN';
    NoOfTools:    ToolCount;
    InitialTools: nil;
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           -2;
    Y0:           -2;
    X1:           195;
    Y1:           280;
    XDividend:    6000;
    YDividend:    6000;
    Divisor:      254;
    Granularity:  1);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  '>Resolution';
    Value:     5;  {600 dpi}
    ParamType: _Explicite;
    Values:    '>75 dpi|>100 dpi|1>50 dpi|>200 dpi|>300 dpi|>600 dpi|12>00 dpi'),
   (DescrStr:  '>Left Margin (mm)';
    Value:     0;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    200),
   (DescrStr:  '>Upper Margin (mm)';
    Value:     0;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    200),
   (DescrStr:  '>Eject Page';
    Value:     Ord (true);
    ParamType: _Explicite;
    Values:    'Off|On'),
   (DescrStr:  '>Compression';
    Value:     Ord (true);
    ParamType: _Explicite;
    Values:    'Off|On'));

// -----------------------------------------------------------------------------

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

// -----------------------------------------------------------------------------

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
begin
  Write := WriteProc;

  WriteStr (#27'*t' + IntStr (Resolutions [Params [Resolution].Value]) + 'R');
                            {Raster Graphics Resolution}
  if Params [LeftMargin].Value > 0 then
    WriteStr (#27'&a' + IntStr (10 * 720 * longint (Params [LeftMargin].Value) div 254)
            + ('H'          {Horizontal Cursor Positioning (Decipoints)}
            + #27'*r1A'))   {Start Raster Graphics - at current cursor position}
  else
    WriteStr (#27'*r0A');   {Start Raster Graphics - at default left margin}
  if Params [UpperMargin].Value > 0 then begin
    WriteStr (#27'*b' + IntStr (10 * Resolutions [Params [Resolution].Value]
                                   * longint (Params [UpperMargin].Value) div 254) + 'Y');
                            {Raster Y Offset}
  end;
end;

PROCEDURE OutputBitmap (BitmapWidth, BitmapHeight, NumOfPlanes: integer; const Planes: Planes_T);
  var Row, Column, ByteCount: integer;
      RowBuffer: array [0..2047] of byte;  {??? ohlidat preteceni!}
begin
  for Row := 0 to BitmapHeight - 1 do begin
    if Boolean (Params [Compression].Value) then begin
      Column := 0;   ByteCount := 0;
      while (Column < BitmapWidth) and (ByteCount < BitmapWidth) do begin
        RowBuffer [ByteCount]     := 0;  {nuluj citac opakovani}
        RowBuffer [ByteCount + 1] := Planes [0]^ [Row]^ [Column];
        Inc (Column);
        while (Column < BitmapWidth)  {jsem stale uvnitr bitmapy}
          and (RowBuffer [ByteCount + 1] = Planes [0]^ [Row]^ [Column])
          and (RowBuffer [ByteCount] < 255)
        do begin
          Inc (RowBuffer [ByteCount]);  {zvys citac opakovani}
          Inc (Column);
        end;
        Inc (ByteCount, 2);
      end;
    end;
    if Boolean (Params [Compression].Value) and (ByteCount < BitmapWidth) then begin
      WriteStr (#27'*b1m' + IntStr (ByteCount) + 'W');  {Transfer Raster Data}
      Write (RowBuffer, ByteCount);
    end
    else begin
      WriteStr (#27'*b0m' + IntStr (BitmapWidth) + 'W');  {Transfer Raster Data}
      Write (Planes [0]^ [Row]^, BitmapWidth);
    end;
  end;
end;

PROCEDURE CloseEquip;
begin
  WriteStr (#27'*rB');      {End Raster Graphics}
  if Boolean (Params [EjectPage].Value) then WriteStr (^L);  {form feed}
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OutputBitmap,
  CloseEquip;

BEGIN
END.

