(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  parametry procedury OutputBitmap zmnny na typ integer (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY PCX_Col;

USES DrvTypes, DLL_Util;

CONST
  ToolCount = 0;
  DriverDescr: DriverDescr_T = (
    Name:         'PCX File (16-Color)';
    MajorVersion: 1;
    MinorVersion: 1;
    Capabilities: [BitMapped, UserTools, ConfigTools];
    NoOfParams:   0;
    FilesName:    'PCX_Col';
    TargetExt:    '.PCX';
    NoOfTools:    ToolCount;
    InitialTools: nil {@ InitToolTable [1]};
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           0;
    Y0:           0;
    X1:           280;
    Y1:           195;
    XDividend:    6000;
    YDividend:    6000;
    Divisor:      254;
    Granularity:  1);

// -----------------------------------------------------------------------------

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetColors: ColorsStr_T;
begin
  Result := 'BGRI';
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := nil;  { @ Params [ParamIndex_T (ParamNumber)]; }
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
{ Params [ParamIndex_T (ParamNumber)].Value := ParamValue; }
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

// -----------------------------------------------------------------------------

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
  const
    StandardPalette: array [0..15, 0..2] of Byte = (
      {black}       ($00, $00, $00),
      {blue}        ($00, $00, $AA),
      {green}       ($00, $AA, $00),
      {cyan}        ($00, $AA, $AA),
      {red}         ($AA, $00, $00),
      {magenta}     ($AA, $00, $AA),
      {brown}       ($AA, $55, $00),
      {light grey}  ($AA, $AA, $AA),
      {dark grey}   ($55, $55, $55),
      {lt blue}     ($55, $55, $FF),
      {lt green}    ($55, $FF, $55),
      {lt cyan}     ($55, $FF, $FF),
      {lt red}      ($FF, $55, $55),
      {lt magenta}  ($FF, $55, $FF),
      {yellow}      ($FF, $FF, $55),
      {white}       ($FF, $FF, $FF)
      );
  var
    Colour, Component: integer;
begin
  X1 := X0+((X1-X0+1+7) and $FFF8)-1; {zaokrouhlit sirku na cele byte}

  Write := WriteProc;
  WriteByte ($0A);   WriteByte ($02);
  WriteByte ($01);   WriteByte ($01);

  WriteWord (X0);   WriteWord (Y0);
  WriteWord (X1);   WriteWord (Y1);

  WriteWord (254 * longint (DriverDescr.XDividend) div (DriverDescr.Divisor * 10));
  WriteWord (254 * longint (DriverDescr.YDividend) div (DriverDescr.Divisor * 10));

  for Colour := 0 to 15 do
    for Component := 0 to 2 do
      WriteByte (StandardPalette [Colour, Component]);

  WriteByte ($00);              {reserved byte}
  WriteByte (4);                {No. of planes}
  WriteWord ((X1-X0+1) div 8);  {No. of bytes per line}
  WriteWord (1);                {Palette interpretation}

  for Colour := 1 to 58 do WriteByte (0);  {58 free bytes}
end;

PROCEDURE OutputBitmap (BitmapWidth, BitmapHeight, NumOfPlanes: integer; const Planes: Planes_T);
  var
    Plane, x, y: integer;
    Count, Last, This: byte;

  procedure WriteBytes;
  begin
    if (Count > 1) or (Last >= $C0) then
      WriteByte ($C0 + Count);
    WriteByte (Last);
    Last := This;
    Count := 1;
  end;

begin {OutputBitmap}
  for y := 0 to BitmapHeight - 1 do begin
    for Plane := 0 to 4 - 1 do begin
      if Plane < NumOfPlanes then
        Last := Planes [Plane]^ [y]^ [0]
      else
        Last := 0;
      Count := 1;
      for x := 1 to BitmapWidth - 1 do begin
        if Plane < NumOfPlanes then
          This := Planes [Plane]^ [y]^ [x]
        else
          This := 0;
        if (Last <> This) or (Count = 63) then
          WriteBytes
        else
          Inc (Count);
      end;
      WriteBytes;
    end;
  end;
end {OutputBitmap};

PROCEDURE CloseEquip;
begin
end;

EXPORTS
  GetDescrPtr440,
  GetColors,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OutputBitmap,
  CloseEquip;

BEGIN
END.

