(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2005 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  vytvoena samostatn jednotka pro formtovn seznamu (X92)
2)  formt vstupnho souboru pizpsoben importu do Excelu (P95)
3)  vstupn soubor u soustek uvd poty pin, resp. slo max. pinu (P95) 
4)  k vrstvm pedvanm do DLL doplnna prvn a posledn vodiv (P96)
--------------------------------------------------------------------------------

DODLAT:
1)  { TODO : doplnit proceduru pro identifikovn verze }

POZNMKY:
1)

----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

unit PL;

interface

uses SysUtils;  // funkce Format

type
  TLayers = set of 0..31;

  TMetricCoords = packed record
                    x, y: Extended;     // v milimetrech
                  end;

  TCompDescr = packed record
                 Name,
                 Value,
                 Pckg:      string [31];
                 Center:    TMetricCoords;
                 Angle:     Integer;
                 Mirrored,
                 Marked:    Boolean;
                 NoOfPins:  Integer;
                 PinLayers,
                 PadLayers: TLayers;
                 Spare1,
                 Spare2,
                 Spare3,
                 Spare4:    Integer;
               end;
  PCompDescr = ^ TCompDescr;

  TBoardDescr = packed record
                  NoOfComps: Integer;
                  Origin:    TMetricCoords;
                  BotLayer,             // slo spodn vnj vodiv vrstvy
                  TopLayer,             // slo horn vnj vodiv vrstvy
                  LayerA,
                  LayerB:    Integer;
                end;

  TWriteProc = procedure (const Str: string);
  TPartlistProc = procedure (var CompDescr: array of PCompDescr; const BoardDescr: TBoardDescr;
                             Write: TWriteProc);

procedure ProcessPartlist (var CompDescr: array of PCompDescr; const BoardDescr: TBoardDescr;
                           Write: TWriteProc);

implementation // ==============================================================

function Underscore (const Str: string): string;
  var i: Integer;
begin
  Result := Str;
  for i := 1 to Length (Result) do
    if Result [i] = ' ' then Result [i] := '_';
end;

procedure ProcessPartlist (var CompDescr: array of PCompDescr; const BoardDescr: TBoardDescr;
                           Write: TWriteProc);
  type
    TSide = (Top, Bottom, Undefined);
  var
    Side: TSide;
    i: Integer;
  const
    MarkStrs: array [Boolean] of string [1] = ('-', '*');
    OrientStrs: array [0..7]  of string [5] = ('R0', 'R90', 'R180', 'R270', 'MR0', 'MR90', 'MR180', 'MR270');
    SideStrs: array [TSide]   of string [3] = ('TOP', 'BOT', '   ');

  function CompSide (Layers: TLayers; Mirrored: Boolean): TSide;
  begin
    with BoardDescr do begin
      Layers := Layers * [BotLayer, TopLayer];
      if Layers = [BotLayer, TopLayer] then     // piny jsou na obou stranch
        if Mirrored then
          Result := Bottom                      // klasick soustka je zrcadlena
        else
          Result := Top
      else if Layers = [TopLayer] then
        Result := Top                           // piny jsou jen shora
      else if Layers = [BotLayer] then
        Result := Bottom                        // piny jen zdola
      else  { Layers = [] }
        Result := Undefined;
    end;
  end {CompSide};

begin
  Write ('Partlist from FORMICA 4.40 / P96   Copyright (C) 1986 - 2005 by Petr Horsk'^M^J);
  DecimalSeparator := '.';
  for Side := Top to Undefined do
    for i := 0 to High (CompDescr) do
      with CompDescr [i]^ do
        if CompSide (PinLayers, Mirrored) = Side then
          Write (Format ('%s'^I'%s'^I'%s'^I'%3d'^I'%s'^I'%8.3f'^I'%8.3f'^I'%s'^I'%s'^M^J,
                         [Underscore (Name),
                          Underscore (Value),
                          Underscore (Pckg),
                          NoOfPins,
                          MarkStrs [Marked],
                          Center.x,
                          Center.y,
                          OrientStrs [Angle div 90],
                          SideStrs [Side]]));
end {ProcessPartlist};

end.
