(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  do funkce OpenLayer doplnn parametr Color (P93)
3)  potlaeno definovn novch clonek pi rotaci (ovlnho i obdlnkovho)
    pjecho bodu stejnho logickho typu (P94)
4)  pro zmnu clonky dle uvn kd G54 (P94)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY RS274X;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    NoOfDigits,
    RepeatedCoordinates,
    EvenParityEnabled);

CONST
  ToolCount = 0;
  DriverDescr: DriverDescr_T = (
    Name:         'RS-274X PH 2004';
    MajorVersion: 4;
    MinorVersion: 2;
    Capabilities: [DrawLines, FlashPads, UserTools, ConfigTools];
    NoOfParams:   0 {Ord (High (ParamIndex_T)) + 1};
    FilesName:    'RS274X';
    TargetExt:    '';
    NoOfTools:    ToolCount;
    InitialTools: nil;
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           0;
    Y0:           0;
    X1:           800;
    Y1:           600;
    XDividend:    1000;
    YDividend:    1000;
    Divisor:      1);

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := nil {@ Params [ParamIndex_T (ParamNumber)]};
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
{ Params [ParamIndex_T (ParamNumber)].Value := ParamValue; }
end;

// -----------------------------------------------------------------------------

CONST
  Digits = 6;
  ApertureOffset = 10;
  NoTool: ToolDescr_T = (DescrStr: '';
                         Shape:    CircleTool;
                         Width:    MaxInt;
                         Height:   MaxInt;
                         Usage:    [];
                         Code:     0);

VAR
  CurrentX, CurrentY: DriverUnits_T;
  CurrentToolCode,
  NewApertureCode:    ToolCode_T;
  CurrentTool,
  RotatedTool:        ToolDescr_T;

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
begin
  Write := WriteProc;
  WriteStr ('G71*'^M^J                  // metrick souadnice
          + 'G75*'^M^J                  // povoleny zporn hodnoty
          + '%FSAX33Y33*%'^M^J          // absolutn formt 3.3
        { + '%OFA0B0*%'^M^J }           // nulov offset
          + '%IPPOS*%'^M^J);            // positivn obraz
  CurrentX        := MaxInt;
  CurrentY        := MaxInt;
  CurrentToolCode := MaxInt;
  NewApertureCode := ApertureOffset;
  CurrentTool     := NoTool;
  RotatedTool     := NoTool;
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

PROCEDURE OpenLayer (Layer: integer; Color: integer = 0);
begin
end;

function Decimal (x: Thousandth_T): string;
begin
  Decimal := IntStr (x div 1000) + '.' + LZIntStr (x mod 1000, 3);
end;

procedure DefineAperture (Shape: ToolShape_T; Width, Height: Thousandth_T;
                          out ToolDescr: ToolDescr_T);
begin
  case Shape of
    CircleTool:   if Width = Height then
                    WriteStr ('%ADD' + IntStr (NewApertureCode)
                            + 'C,' + Decimal (Width) + '*%'^M^J)
                  else
                    WriteStr ('%ADD' + IntStr (NewApertureCode)
                            + 'O,' + Decimal (Width)
                            + 'X'  + Decimal (Height) + '*%'^M^J);
    SquareTool:   WriteStr   ('%ADD' + IntStr (NewApertureCode)
                            + 'R,' + Decimal (Width)
                            + 'X'  + Decimal (Height) + '*%'^M^J);
    AnnulusTool:  ;
    ThermalTool:  ;
  end;
  ToolDescr        := NoTool;
  ToolDescr.Shape  := Shape;
  ToolDescr.Width  := Width;
  ToolDescr.Height := Height;
  ToolDescr.Code   := NewApertureCode;
  Inc (NewApertureCode);
end;

procedure ChangeToolCode (Code: ToolCode_T);
begin
  if Code <> CurrentToolCode then begin
    WriteStr ('G54D' + IntStr (Code) + '*'^M^J);
    CurrentToolCode := Code;
  end;
end;

procedure ChangeAperture (ToolNumber: ToolNumber_T);
begin
  with WorkToolTable [ToolNumber] do
    if      (Shape = CurrentTool.Shape)
      and   (Width = CurrentTool.Width) and (Height = CurrentTool.Height)
    then
      ChangeToolCode (CurrentTool.Code)
    else if (Shape = RotatedTool.Shape)
      and   (Width = RotatedTool.Width) and (Height = RotatedTool.Height)
    then
      ChangeToolCode (RotatedTool.Code)
    else begin
      DefineAperture (  Shape, Width, Height, CurrentTool);
      if Width <> Height then
        DefineAperture (Shape, Height, Width, RotatedTool);
      ChangeToolCode (CurrentTool.Code);
    end;
end;

{ Procedura ChangeAperture byla zmnna, aby se tak potlailo definovn novch
  clonek pi rotaci (ovlnho i obdlnkovho) pjecho bodu stle stejnho
  logickho typu.  Cenou za to je, e z dvojice definovanch clonek nemus bt
  druh vbec uita.

procedure ChangeAperture (ToolNumber: ToolNumber_T);
begin
  with WorkToolTable [ToolNumber] do
    if   (Shape <> CurrentTool.Shape)
      or (Width <> CurrentTool.Width) or (Height <> CurrentTool.Height)
    then begin
      case Shape of
        CircleTool:   if Width = Height then
                        WriteStr ('%ADD' + IntStr (CurrentToolCode)
                                  + 'C,'  + Decimal (Width) + '*%'^M^J)
                      else
                        WriteStr ('%ADD' + IntStr (CurrentToolCode)
                                  + 'O,'  + Decimal (Width)
                                  + 'X'   + Decimal (Height) + '*%'^M^J);
        SquareTool:   WriteStr   ('%ADD' + IntStr (CurrentToolCode)
                                  + 'R,'  + Decimal (Width)
                                  + 'X'   + Decimal (Height) + '*%'^M^J);
        AnnulusTool:  ;
        ThermalTool:  ;
      end;
      WriteStr ('D' + IntStr (CurrentToolCode) + '*'^M^J);
      CurrentTool := WorkToolTable [ToolNumber];
      Inc (CurrentToolCode);
    end;
end;                                                                           }

procedure OutputCoordinates (X, Y: DriverUnits_T);
begin
  WriteStr ('X' + LZIntStr (X, 6) + 'Y' + LZIntStr (Y, 6));
  CurrentX := X;
  CurrentY := Y;
end;

PROCEDURE DrawLine (X0, Y0, X1, Y1: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  ChangeAperture (ToolNumber);
  if (X0 <> CurrentX) or (Y0 <> CurrentY) then begin
    OutputCoordinates (X0, Y0);
    WriteStr ('D02*'^M^J);
  end;
  OutputCoordinates (X1, Y1);
  WriteStr ('D01*'^M^J);
end;

PROCEDURE FlashPad (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  ChangeAperture (ToolNumber);
  OutputCoordinates (X, Y);
  WriteStr ('D03*'^M^J);
end;

PROCEDURE DrillHole (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  FlashPad (X, Y, ToolNumber);
end;

PROCEDURE CloseEquip;
begin
  WriteStr ('M02*'^M^J);
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OpenLayer,
  DrawLine,
  FlashPad,
  DrillHole,
  CloseEquip;

BEGIN
END.
